#include "mbed.h"
#include <cstdint>
#include <string.h>
#include "MOD24_NRF.h"

//Module RF
#define WAIT_TIME_MS 500 

UnbufferedSerial    my_pc(USBTX, USBRX);
char        charStr[128];

// RF Transmission of data
#define TRANSFER_SIZE   1
nRF24L01P       nRF24_mod(D11, D12, D13, PA_12, PA_11, PB_12);
// MOSI, MISO, SCK, CSN, CE, IRQ
//char        dataToSend[TRANSFER_SIZE] = {1, 50, 60, 128, 36, 66, 255, 255};
char        dataReceived[TRANSFER_SIZE] = {0};



//Entrées/ Sorties Moteurs
//2 moteurs
PwmOut moteur_EN(PB_3);  // (enable)
//Moteur 2
PwmOut moteur_3A(PB_13);  
PwmOut moteur_4A(PB_14);  //Avancer/Reculer
//Moteur 3
PwmOut moteur_1A(PC_9);  
PwmOut moteur_2A(PB_8);  //Avancer/Reculer

//Positions:
InterruptIn moteur_B2(PB_1);
InterruptIn moteur_A2(PB_2);
InterruptIn moteur_B3(PB_10);
InterruptIn moteur_A3(PB_4);

//Variables Moteur
double rc2=0;
double rc3=0; //Rapport cyclique : pilotage vitesse

int C2=0; // compteur front montant moteur 2
int C3=0; // compteur front montant moteur 3
int Cc;

int e2 =0;
int e3 = 0;  //Erreurs
int e_int_2 = 0;
int e_int_3 = 0;
double Kp2 = 0.001;
double Kp3 = 0.001;
double Ki2 = 0.0001;
double Ki3 = 0.0001;

Ticker toggle_V_ticker;   //Déclaration ticker

//Déclaration des fonctions moteur
void compteur2(void);
void compteur3(void);
void toggle_V(void);

//Déclarations fonctions module
// Initialization function for the BT nRF24L01 module
//  frequency in MHz
void        initNRF24(int frequency);
// Receiving function for the BT nRF24L01 module
//  data : array of data received by the module
//  return the number of bytes received
uint8_t     receiveNRF24(char *data);
// Transmitting function for the BT nRF24L01 module
//  data : array of data to transmit
void        transmitNRF24(char *data);

int main()
{
    printf("Veronica");

    //Module RF
    my_pc.baud(115200);
    sprintf(charStr, "Mbed OS %d.%d.%d.\r\n", MBED_MAJOR_VERSION, MBED_MINOR_VERSION, MBED_PATCH_VERSION);
    my_pc.write(charStr, strlen(charStr));

    initNRF24(2450);

// Initialisation moteurs
    moteur_EN.period_ms(10);
    moteur_EN.write(0);
    //Moteur 2
    moteur_3A.period_ms(10);
    moteur_4A.period_ms(10);
    moteur_3A.write(0);
    
    //Moteur 3
    moteur_1A.period_ms(10);
    moteur_2A.period_ms(10);
    moteur_2A.write(0);



    moteur_A2.rise(&compteur2);
    moteur_A3.rise(&compteur3);

    //Affichage des valeurs des compteurs 
    toggle_V_ticker.attach(&toggle_V, 0.5); //Asservissement toutes les 0.5s

    while (true) 
    {
        //dataToSend[0]++;
        //transmitNRF24(dataToSend);
        uint8_t  nb_data = receiveNRF24(dataReceived);


        if(nb_data != 0){
            for(int i = 0; i < nb_data; i++){
                sprintf(charStr, "%d ", dataReceived[i]); //Reçoit la valeur par RF
                //my_pc.write(charStr, strlen(charStr));     
                Cc = dataReceived[i];    // stocke la valeur dans Cc (Commande)
                printf("valeur louis : %d\n",Cc);     //Vérif  
            }
            sprintf(charStr, "\r\n");
            my_pc.write(charStr, strlen(charStr)); 
        }
        thread_sleep_for(WAIT_TIME_MS);

        //Alimentée en 12,2V par batterie
        //Marche moteur
        moteur_EN.write(1);

        //Moteur 2 : Avance (3=0) ou recule (4=0)
        moteur_3A.write(0);
        
         //Moteur 2 : Avance (2=0) ou recule (1=0)
        moteur_2A.write(0);
       

    }
        

}

void compteur2(void)
{
    C2=C2+1;    
}

void compteur3(void)
{
    C3=C3+1;
}

void toggle_V(void)
{
    //Asservissement vitesse
    e2 = Cc-C2;
    e3 = Cc-C3;
    e_int_2+=e2;
    e_int_3+=e3;
    if (e2 != 0)
    {
        rc2 = rc2+(Kp2*e2+Ki2*e_int_2);
    }


    if (e3 != 0)
    {
        rc3 = rc3+(Kp3*e3+Ki3*e_int_3);
    }

    if (rc3<=1)
    {
        moteur_1A.write(rc3); //Asservissement moteur 3 
    }
    else
    {
        //printf("erreur !");
        rc3=1;
    }
    if (rc2<=1)
    {
        moteur_4A.write(rc2); //Asservissement moteur 2
        
    }
    else
    {
        //printf("erreur !");
        rc2=1;
    }
    
    printf("Compteur 2 = %d\n",C2);
    printf("Compteur 3 = %d\n",C3);
    printf("Erreur 2 = %d\n",e2);
    printf("Erreur 3 = %d\n",e3);
    printf("Erreur int 3 = %d\n",e_int_3);

    C2=0;
    C3=0;

}

// Initialization function for the BT nRF24L01 module
void initNRF24(int frequency){
    nRF24_mod.powerUp();
    wait_us(100000);
    nRF24_mod.setAirDataRate(NRF24L01P_DATARATE_250_KBPS);
    nRF24_mod.setRfFrequency(frequency);
    wait_us(1000000);

    sprintf(charStr, "nRF24L01+ Frequency    : %d MHz\r\n",  nRF24_mod.getRfFrequency());
    my_pc.write(charStr, strlen(charStr)); 
    sprintf(charStr, "nRF24L01+ Data Rate    : %d kbps\r\n", nRF24_mod.getAirDataRate());
    my_pc.write(charStr, strlen(charStr)); 

    nRF24_mod.setTransferSize( TRANSFER_SIZE );
    nRF24_mod.setReceiveMode();
    nRF24_mod.enable();
}

// Receiving function for the BT nRF24L01 module
uint8_t receiveNRF24(char *data){
    uint8_t        rxDataCnt = 0;
    if ( nRF24_mod.readable() ) {
        // Read the data into the receive buffer
        rxDataCnt = nRF24_mod.read( NRF24L01P_PIPE_P0, data, TRANSFER_SIZE);
    }
    return rxDataCnt;
}

// Transmitting function for the BT nRF24L01 module
//  data : array of data to transmit
void        transmitNRF24(char *data){    
    nRF24_mod.write( NRF24L01P_PIPE_P0, data, TRANSFER_SIZE );
}